// AppUpdateDlg.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "AppUpdateDlg.h"
#include "KhanInet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define CHECK_UPDATE_STR _T("Check For Updates")
#define DO_UPDATE_STR _T("Install Update")
#define CANCEL_UPDATE_STR _T("Cancel Update")

/////////////////////////////////////////////////////////////////////////////
// CAppUpdateDlg dialog


CAppUpdateDlg::CAppUpdateDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAppUpdateDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CAppUpdateDlg)
	m_strStatus = _T("");
	//}}AFX_DATA_INIT

	// Set members.
	m_doUpdate = FALSE;
	m_exeSz = 0;
}


void CAppUpdateDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAppUpdateDlg)
	DDX_Control(pDX, IDC_BUTTON_UPDATE, m_ctrlUpdate);
	DDX_Control(pDX, IDOK, m_ctrlDone);
	DDX_Control(pDX, IDC_EDIT_STATUS, m_ctrlStatus);
	DDX_Text(pDX, IDC_EDIT_STATUS, m_strStatus);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAppUpdateDlg, CDialog)
	//{{AFX_MSG_MAP(CAppUpdateDlg)
	ON_BN_CLICKED(IDC_BUTTON_UPDATE, OnButtonUpdate)
	//}}AFX_MSG_MAP

	ON_MESSAGE (WM_USER_STATUS_STRING,	OnStatusString)
	ON_MESSAGE (WM_USER_FINISHED,		OnAppUpdateFinished)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAppUpdateDlg message handlers

BOOL CAppUpdateDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return FALSE;

	// Set initial text per user status.
	// If hosed at TUX (defined in StdAfx.h).
	if(TUX_HOST)
	{
		if(!pDoc->IsAdmin())
			m_strStatus = _T("Updates are obtained from:\r\n")
						  _T("   www.khandesktop.tuxfamily.org.\r\n")
						  _T("You will need elevated administrative \r\n")
						  _T("rights to complete the update.\r\n");
		else
			m_strStatus = _T("Updates are obtained from:\r\n")
						  _T("   www.khandesktop.tuxfamily.org.\r\n")
						  _T("Note administrative rights are \r\n")
						  _T("required to complete the update.\r\n");
	}
	else // SF
	{
		if(!pDoc->IsAdmin())
			m_strStatus = _T("Updates are obtained from SourceForge.net.\r\n")
						  _T("You will need elevated administrative \r\n")
						  _T("rights to complete the update.\r\n");
		else
			m_strStatus = _T("Updates are obtained from SourceForge.net.\r\n")
						  _T("Note administrative rights are \r\n")
						  _T("required to complete the update.\r\n");
	}
		
	((CApp*)AfxGetApp())->SetThreadMessage(m_strStatus);

	// Set button text
	m_ctrlUpdate.SetWindowText(CHECK_UPDATE_STR);

	// Only update once.
	if(pDoc->m_appUpdateInstalled)
	{
		m_strStatus = _T("An update has been installed.\r\n")
					  _T("Please restart Khan Desktop to activate.");
		m_ctrlUpdate.EnableWindow(FALSE);
	}

	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAppUpdateDlg::OnButtonUpdate() 
{
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	CString str;
	m_ctrlUpdate.GetWindowText(str);

	// Run the App Update Thread - check for updates.
	if(str == CHECK_UPDATE_STR)
	{
		// Set for cancelling.
		m_ctrlUpdate.SetWindowText(CANCEL_UPDATE_STR);

		// Disable Done button - user must press cancel first.
		m_ctrlDone.EnableWindow(FALSE);

		if(!pDoc->UpdateApplication(GetSafeHwnd(), TRUE))
		{
			m_strStatus += _T("\r\nCannot update.");
			((CApp*)AfxGetApp())->SetThreadMessage(m_strStatus);
			UpdateData(FALSE);
			m_ctrlStatus.LineScroll(m_ctrlStatus.GetLineCount());
		}
	}

	else if(str == DO_UPDATE_STR)
	{
		// Copy exe.
		if(!pDoc->UpdateApplication(GetSafeHwnd(), FALSE))
		{
		}

		// Only update once.
		if(pDoc->m_appUpdateInstalled)
			m_ctrlUpdate.EnableWindow(FALSE);
	}
	else if(str == CANCEL_UPDATE_STR)
	{
		// Cancel the update.
		pDoc->CancelAppUpdate();
	}
}


// Called from KhanInet via SendMessage.
LRESULT CAppUpdateDlg::OnStatusString(WPARAM wParam, LPARAM lParam)	// WM_USER_STATUS_STRING
{
	// Get message string from CApp.
	CString str;
	if(	((CApp*)AfxGetApp())->GetThreadMessage(str) )
	{
		TRACE(_T("CAppUpdateDlg::OnStatusString\n"));

		// Set the CEdit
		m_ctrlStatus.SetWindowText(str);
		m_ctrlStatus.UpdateWindow();
		m_ctrlStatus.LineScroll(m_ctrlStatus.GetLineCount());
		m_strStatus = str;
	}
    return 0;
}

// Called from KhanInet via SendMessage.
LRESULT CAppUpdateDlg::OnAppUpdateFinished(WPARAM wParam, LPARAM lParam) // WM_USER_FINISHED
{
	// Enable Done button.
	m_ctrlDone.EnableWindow(TRUE);

	// Get CDoc.
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		m_ctrlUpdate.EnableWindow(FALSE);
		return 0;
	}

	// New exe has been stored temporarilly in app data folder.
	// User can install if desired via CDoc.
	if(lParam != 0)
	{
		// Set button text
		m_ctrlUpdate.SetWindowText(DO_UPDATE_STR);

		// If not an admin, update cannot be installed.
		// Info user, clean up, and bail.

		// Remind user of admin rights.
		if(!pDoc->IsAdmin())
		{
			m_strStatus += _T("\r\n\r\nNote you will need to elevate to an\r\n")
						   _T("administrator to install the Update.");
			((CApp*)AfxGetApp())->SetThreadMessage(m_strStatus);
			UpdateData(FALSE);
			m_ctrlStatus.LineScroll(m_ctrlStatus.GetLineCount());
		}
		else
		{
			m_strStatus += _T("\r\n\r\nNote the update must be installed\r\n")
						   _T("using administrator privileges.");
			((CApp*)AfxGetApp())->SetThreadMessage(m_strStatus);
			UpdateData(FALSE);
			m_ctrlStatus.LineScroll(m_ctrlStatus.GetLineCount());
		}
		return 0;
	}

	// Else canceled or an error.
	else
	{
		// Set button text
		m_ctrlUpdate.SetWindowText(CHECK_UPDATE_STR);

		// If update was canceled or a error, clean any
		// left over temp files.
		pDoc->CleanCanceledAppUpdate();
		return 0;
	}
}



