// FontSetup.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "FontSetup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MIN_HEIGHT 10
#define MAX_HEIGHT 40
#define MIN_WEIGHT 100
#define MAX_WEIGHT 900

/////////////////////////////////////////////////////////////////////////////
// CFontSetup dialog


CFontSetup::CFontSetup(CWnd* pParent /*=NULL*/)
	: CDialog(CFontSetup::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFontSetup)
	m_strCurFontName = _T("");
	//}}AFX_DATA_INIT

	ZeroMemory(&m_lf, sizeof(LOGFONT));
	m_lf.lfHeight = -16;
	//wcscpy(m_lf.lfFaceName, _T("Arial"));
	StringCbCopy(m_lf.lfFaceName, 6 * sizeof(TCHAR), _T("Arial")); 
}


void CFontSetup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFontSetup)
	DDX_Control(pDX, IDC_FONT_SAMPLE, m_ctrlSample);
	DDX_Control(pDX, IDC_COMBO_WEIGHT, m_ctrlWeight);
	DDX_Control(pDX, IDC_COMBO_HEIGHT, m_ctrlHeight);
	DDX_Control(pDX, IDC_LIST_FONTS, m_ctrlListBox);
	DDX_Text(pDX, IDC_EDIT_CUR_FONT, m_strCurFontName);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFontSetup, CDialog)
	//{{AFX_MSG_MAP(CFontSetup)
	ON_LBN_SELCHANGE(IDC_LIST_FONTS, OnSelchangeListFonts)
	ON_CBN_KILLFOCUS(IDC_COMBO_HEIGHT, OnKillfocusHeight)
	ON_CBN_SELCHANGE(IDC_COMBO_HEIGHT, OnSelchangeHeight)
	ON_CBN_KILLFOCUS(IDC_COMBO_WEIGHT, OnKillfocusWeight)
	ON_CBN_SELCHANGE(IDC_COMBO_WEIGHT, OnSelchangeWeight)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFontSetup message handlers

BOOL CFontSetup::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Init font names, height and weight combos.
	FillListBox();
	FillCombos();

	// Set name str.
	m_strCurFontName = m_lf.lfFaceName;

	// Set font name list sel.
	m_ctrlListBox.SelectString(0, m_strCurFontName);

	// Set height and weight.
	SetHeightSel(m_lf.lfHeight);
	SetWeightSel(m_lf.lfWeight);

	// Draw sample.
	SetSampleText();
	UpdateData(FALSE);

	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CFontSetup::FillCombos()
{
	m_ctrlWeight.AddString(_T("100"));
	m_ctrlWeight.AddString(_T("200"));
	m_ctrlWeight.AddString(_T("300"));
	m_ctrlWeight.AddString(_T("400"));
	m_ctrlWeight.AddString(_T("500"));
	m_ctrlWeight.AddString(_T("600"));
	m_ctrlWeight.AddString(_T("700"));
	m_ctrlWeight.AddString(_T("800"));
	m_ctrlWeight.AddString(_T("900"));

	CString s;
	for(int i = MIN_HEIGHT; i <= MAX_HEIGHT; i++)
	{
		s.Format(_T("%d"), i);
		m_ctrlHeight.AddString(s);
	}
}

void CFontSetup::FillListBox()
{
    m_ctrlListBox.ResetContent();

    CClientDC dc (this);
    ::EnumFontFamilies ((HDC)dc, NULL, (FONTENUMPROC) EnumFontFamProc,
        (LPARAM) this);
}

void CFontSetup::SetSampleText()
{
	if((HFONT)m_fontSample)
		m_fontSample.DeleteObject();

	m_fontSample.CreateFontIndirect(&m_lf);
	m_ctrlSample.SetFont(&m_fontSample, FALSE);

	CRect r;
	m_ctrlSample.SetWindowText(_T("Khan Desktop"));

}

int CALLBACK CFontSetup::EnumFontFamProc (ENUMLOGFONT* lpelf, NEWTEXTMETRIC* lpntm, 
										  int nFontType, LPARAM lParam)
{
    CFontSetup * pWnd = (CFontSetup*) lParam;

	if(nFontType & TRUETYPE_FONTTYPE)
	{
		CString str = lpelf->elfLogFont.lfFaceName;
		pWnd->m_ctrlListBox.AddString(str);
	}
	return 1;

/*
    CFontSetup * pWnd = (CFontSetup*) lParam;

//	if ((pWnd->m_wndCheckBox.GetCheck () == BST_UNCHECKED) ||
//	(nFontType & TRUETYPE_FONTTYPE))// 
//	{
	CString str = lpelf->elfLogFont.lfFaceName;
//	if(lpelf->elfLogFont.lfPitchAndFamily & FIXED_PITCH)//FF_DECORATIVE)//FF_SCRIPT)//FF_MODERN)
//	str += "*";
	pWnd->m_ctrlListBox.AddString(str);
	//  pWnd->m_wndListBox.AddString (lpelf->elfLogFont.lfFaceName);
//	}
	return 1;
*/
}

// Set font pre DoModal().
void CFontSetup::SetCurFont(const LOGFONT & lf)
{
	m_lf = lf;
}

// Font name changed.
void CFontSetup::OnSelchangeListFonts() 
{
	CString s;
	int i = m_ctrlListBox.GetCurSel();
	if ( i != LB_ERR && m_ctrlListBox.GetCount() )
	{
		m_ctrlListBox.GetText(i, s);
		//wcscpy(m_lf.lfFaceName, s);
		StringCbCopy(m_lf.lfFaceName, (s.GetLength() + 1) * sizeof(TCHAR), s);
		SetSampleText();
		m_strCurFontName = m_lf.lfFaceName;
		UpdateData(FALSE);
	}	
}

//////////////////////////////////////////////////////////////
// Height

// Set font height.
void CFontSetup::SetHeightSel(int n, BOOL drawSample)
{
	n = abs(n);
	if(n < MIN_HEIGHT)
		n = MIN_HEIGHT;
	else if(n > MAX_HEIGHT)
		n = MAX_HEIGHT;

	CString s;
	s.Format(_T("%d"), n);
	m_ctrlHeight.SelectString(0, s);

	// Set m_lf
	m_lf.lfHeight = -n;

	if(drawSample)
		SetSampleText();
}

// Height focus lost.
void CFontSetup::OnKillfocusHeight() 
{
	CString s;
	m_ctrlHeight.GetWindowText(s);
	s.TrimLeft();
	s.TrimRight();
	int n = _ttoi(s);
	SetHeightSel(n, TRUE);
}

// Height Selection.
void CFontSetup::OnSelchangeHeight() 
{
	int i = m_ctrlHeight.GetCurSel();
	if(i == LB_ERR)
		return;
	CString s;

	m_ctrlHeight.GetLBText(i, s);
	int n = _ttoi(s);
	SetHeightSel(n, TRUE);	
}

//////////////////////////////////////////////////////////////
// Weight

// Set font height.
void CFontSetup::SetWeightSel(int n, BOOL drawSample)
{
	n = abs(n);
	if(n < MIN_WEIGHT)
		n = MIN_WEIGHT;
	else if(n > MAX_WEIGHT)
		n = MAX_WEIGHT;
	else
	{
		// Even up
		int rem = n % 100;
		n -= rem;
	}

	CString s;
	s.Format(_T("%d"), n);
	m_ctrlWeight.SelectString(0, s);

	// Set m_lf
	m_lf.lfWeight = n;

	if(drawSample)
		SetSampleText();
}

void CFontSetup::OnKillfocusWeight() 
{
	CString s;
	m_ctrlWeight.GetWindowText(s);
	s.TrimLeft();
	s.TrimRight();
	int n = _ttoi(s);
	SetWeightSel(n, TRUE);	
}

void CFontSetup::OnSelchangeWeight() 
{
	int i = m_ctrlWeight.GetCurSel();
	if(i == LB_ERR)
		return;
	CString s;

	m_ctrlWeight.GetLBText(i, s);
	int n = _ttoi(s);
	SetWeightSel(n, TRUE);	
}
