
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/



// KhanInet.cpp 
//

#include "Stdafx.h"
#include "Khan.h"
#include "KhanInet.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define UPDATE_START_STR _T("khandesktopupdate")


/////////////////////////////////////////////////////////////////////////////////////
// Class CKnanInet

CKnanInet::~CKnanInet()
{
}


// Constructor for Khan Update an Video File Download.
CKnanInet::CKnanInet(THREADPARAM & param)

			 : CInternetSession(_T("Khan Desktop Application"), // App name
								1,								// dwContext: 1 = use callback
								PRE_CONFIG_INTERNET_ACCESS)
																// dwFlags INTERNET_FLAG_ASYNC			 
{
	// Enable status callbacks
	EnableStatusCallback(TRUE);

	// Save params to member.
	m_params = param;
}

///////////////////////////////////
// Get the Khan Library JSON data.
// Connects to:
//	www.khanacademy.org/api/v1/topictree
//
BOOL CKnanInet::GetKhanLibraryData(CString tempFilePath, int progBytes)
{
	// Prepare.
	CString server		= _T("www.khanacademy.org");
	CString object		= _T("api/v1/topictree");
	CString headers		= _T("Accept:application/json, text/plain\r\n");// add r 0.9.0
	CString verb		= _T("GET");	// HTTP_VERB_GET 1 
	INTERNET_PORT port	= 80;

	DWORD flags =	INTERNET_FLAG_TRANSFER_BINARY  |
					INTERNET_FLAG_EXISTING_CONNECT |					  
					INTERNET_FLAG_NO_AUTO_REDIRECT | 
					INTERNET_FLAG_DONT_CACHE;

	SetOption(INTERNET_OPTION_CONNECT_TIMEOUT, 30000);
	SetOption(INTERNET_OPTION_RECEIVE_TIMEOUT, 60000);	// v0.8.4 - 45 sec typ
	SetOption(INTERNET_OPTION_CONNECT_BACKOFF, 1000);
	SetOption(INTERNET_OPTION_CONNECT_RETRIES, 5);		// default 5

	CHttpConnection * pServer = NULL;   
	CHttpFile* pFile = NULL;
	DWORD dwRet = 0;
	BOOL failed = FALSE;
	CString msg;

	try 
	{
		// Derived CInternetSession Server.
		pServer = GetHttpConnection(server, port);

		pFile = pServer->OpenRequest(verb, object, NULL, 
									 1, NULL, NULL, flags);

		pFile->AddRequestHeaders(headers);
		pFile->SendRequest();				// INET.CPP HINTERNET hFile
		pFile->QueryInfoStatusCode(dwRet);

		// Connected OK.
		if(dwRet == HTTP_STATUS_OK)
		{
			const int bufSz = 4096;
			char	buf[bufSz];
			UINT	nRead		= 0;
			UINT	total		= 0;
			int		nLoops		= 0; // dbg
			int		progChunk	= 0;
			BOOL	updateStatus = TRUE;

			// KB per sec.
			DWORD	startTime, endTime;
			double	KBps;
			int nReadKBps = 0;			
			startTime = GetTickCount();

			// File for writing.
			CFile datafile(tempFilePath, CFile::modeCreate |
										 CFile::modeWrite  |
										 CFile::typeBinary );

			// Get chunks.
			while((nRead = pFile->Read(buf, bufSz)) > 0)
			{
				if(updateStatus)
				{
					ShowStatus(_T("Downloading Khan Data..."));
					ShowStatus(_T("Please wait."));
					updateStatus = FALSE;
				}

				datafile.Write(buf, nRead);
				nLoops ++;
				total += nRead;

				// Display KBps - once a second.
				nReadKBps += nRead;
				endTime = GetTickCount();
				if(endTime > startTime + 1000)
				{
					KBps = endTime - startTime;				
					// 1 KBps (KB/s) = 1,000 bytes per second;
					KBps = (double)(nReadKBps / KBps);
					SetKBps(KBps);
					
					// New start time for next KBps.
					startTime = endTime;
					nReadKBps = 0;
				}

				// Update dialog's progress ctrl.
				progChunk += nRead;
				if(progChunk > progBytes)
				{
					SetProgressPos(total / progBytes);
					progChunk = 0;
				}

				// User can abort during download.
				if(::WaitForSingleObject(m_params.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
				{
					// Clean up.
					datafile.Close();
					if(pFile)
					{
						pFile->Close();
						delete pFile;
						pFile = NULL;
					}
					if(pServer)
					{
						pServer->Close();
						delete pServer;
						pServer = NULL;
					}
					Close();

					// Post Status msg.
					msg = _T("Canceling Update.");
					ShowStatus(msg);

					// Gone.
					return FALSE;
				}
			}
			datafile.Close();

			// Update status msg and progress bar.
			double dTotal = (double)total;
			double mb = 1024 * 1024;
			double kb = 1024;
			if(total > 1024 * 1024)
				msg.Format(_T("Download succeded!  %.2f MB recieved."), (dTotal / mb) ); 
			else if(total > 1024)
				msg.Format(_T("Download succeded!  %.2f KB recieved."), (dTotal / kb) ); 
			else
				msg.Format(_T("Download succeded!  %d Bytes recieved."), total ); 
			ShowStatus(msg);
		}

		// Else error - dwRet != HTTP_STATUS_OK.
		else
		{
			failed = TRUE;
			// report it
		}

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		if(pServer)
		{
			pServer->Close();
			delete pServer;
			pServer = NULL;
		}
		// Close the connection.
		Close();
	}

     // Catch any exceptions from WinINet
	catch(CInternetException * pEx) 
	{
		TCHAR szErr[1024];
        if(!pEx->GetErrorMessage(szErr, 1024))
			//wcscpy(szErr, _T("Unknown error."));
			StringCbCopy(szErr, 15 * sizeof(TCHAR), _T("Unknown error."));

		TRACE(_T("CKnanInet GetKhanLibraryData File transfer failed!! - %s"),szErr);      
		pEx->Delete();

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		if(pServer)
		{
			pServer->Close();
			delete pServer;
			pServer = NULL;
		}
		// Derived
		Close(); 

		// Info user of error.
		msg = szErr;
		msg.TrimRight();	// has a newline
		msg += _T(". ");	// beautify
		ShowStatus(msg);
		if(msg == _T("The server name or address could not be resolved. "))
		{
			msg = _T("Please ensure you are connected to the internet.");
			ShowStatus(msg);
		}
		msg = _T("The update failed.");
		ShowStatus(msg);

		return FALSE;
	}

	// Catch other exceptions.
	// Not tested.
	// Fix occasional fail @ SendRequest() - v0.9.0
	catch(CException * pEx2)
	{
		TCHAR szErr[1024];
        if(!pEx2->GetErrorMessage(szErr, 1024))
			//wcscpy(szErr,_T("Unknown error 2."));
			StringCbCopy(szErr, 17 * sizeof(TCHAR), _T("Unknown error 2."));

		TRACE(_T("CKnanInet GetKhanLibraryData File transfer failed - unknown exception - %s"),szErr);      
		pEx2->Delete();

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		if(pServer)
		{
			pServer->Close();
			delete pServer;
			pServer = NULL;
		}
		// Derived
		Close(); 

		// Info user of error.
		msg = szErr;
		msg.TrimRight();	// has a newline
		msg += _T(". ");	// beautify
		ShowStatus(msg);
		msg = _T("The update failed - program error 2.");
		ShowStatus(msg);

		return FALSE;	
	} 

	// Final Inet message.
	msg.Format(_T("Khan Web Server returned code: %d"), dwRet);
	ShowStatus(msg);

	// Finish Success.
	if(!failed)
	{
		msg.Format(_T("Khan Update data downloaded successfully."));
		ShowStatus(msg);

		return TRUE;
	}
	// Some error.
	else
	{
		msg.Format(_T("Khan Update download error."));
		ShowStatus(msg);

		return FALSE;
	}
	return FALSE;
}


// Download Video files.
// Typical url: http://s3.amazonaws.com/KA-youtube-converted/frBJEYvyd-8.mp4/frBJEYvyd-8.mp4
BOOL CKnanInet::GetVideoFiles(CString & errMsg)
{
	CString headers	= _T("Accept: */*\r\n");
	DWORD flags =	INTERNET_FLAG_TRANSFER_BINARY  |
					INTERNET_FLAG_EXISTING_CONNECT |					  
					INTERNET_FLAG_NO_AUTO_REDIRECT | 
					INTERNET_FLAG_DONT_CACHE;

	// Set derived CInternetSession session options.
	SetOption(INTERNET_OPTION_CONNECT_TIMEOUT,	30000);
	SetOption(INTERNET_OPTION_RECEIVE_TIMEOUT,	60000);		// test fix error 12002
	SetOption(INTERNET_OPTION_SEND_TIMEOUT,		60000);		// test fix error 12002
	SetOption(INTERNET_OPTION_CONNECT_RETRIES,	1);
	//SetOption(INTERNET_OPTION_CONNECT_BACKOFF, 1000);		// Not implemented

	// Ensure valid params.
	if(!m_params.videoAr			||		// Array of video file paths
	   !m_params.videoAr->GetSize() || 
	   !m_params.path				||		// Storage path
	    m_params.path->IsEmpty()	)
	{
		errMsg = _T("Error downloading video files.");
		ShowStatus(errMsg);
		TRACE(_T("CKnanInet GetVideoFiles invalid params\n"));
		return FALSE;
	}

	// Get number of files to download.
	int sz = m_params.videoAr->GetSize();

	// Set the Total Progress range.
	SetProgressRange(sz * 10);
	SetProgressPos(0);

	CHttpFile * pFile = NULL;
	CFile destfile;
	DWORD dwRet = 0;
	CString msg, s, destPath;
	const int bufSz = 4096;
	char buf[bufSz];
	int nRead, nTotalRead, nReadKBps;
	int step, stepSz, vidProgStep = 0;
	int i = 0;
	DWORD startTime, endTime;
	double KBps;
	int badFiles = 0;

	try 
	{
		// Get files.
		for(i = 0; i < sz; i++)
		{
			// Get a ref.
			CVideo & video = m_params.videoAr->GetAt(i);

			// Get the video file path.
			if(!video.GetShortFileName(s))
			{
				// dud video?
				// mp4	"http://s3.amazonaws.com/KA-youtube-converted/xkg7370cpjs.mp4/xkg7370cpjs.mp4"	ATL::CStringT<wchar_t,StrTraitMFC_DLL<wchar_t,ATL::ChTraitsCRT<wchar_t> > >

				continue;
			}

			// Construct storage path.
			destPath = *m_params.path;
			destPath += _T('\\');
			destPath += s;

			// Open URL.
			pFile = (CHttpFile*)OpenURL(video.mp4,
										1, 
										flags,
										headers, 
										-1);
			// Error.
			if(!pFile)
			{
				errMsg = _T("Error connecting to internet.");
				ShowStatus(errMsg);
				TRACE(_T("CKnanInet GetVideoFiles pFile null\n"));
				Close();
				return FALSE;
			}

			// File for writing.
			CFileException ex;
			if(!destfile.Open(destPath, CFile::modeCreate |		// overwrites any existing
										CFile::modeWrite  |
										CFile::typeBinary,
										&ex				  ))
			{
				TCHAR szError[1024];
				ex.GetErrorMessage(szError, 1024);
				errMsg.Format(_T("Error opening Video file\n%s."), szError);
				ShowStatus(errMsg);
				TRACE(_T("CKnanInet GetVideoFiles can't open destFile\n"));
				if(pFile)
				{
					pFile->Close();
					delete pFile;
					pFile = NULL;
				}
				Close();
				return FALSE;
			}

			// Update status msg.
			msg.Format(_T("Downloading %d of %d: Please wait."), 
							i + 1, sz); 
			ShowStatus(msg);

			// Attempt to get the file length - not guaranteed.
			DWORD len, len1, len2;
			//
			// Try QueryInfo and GetLength().
			pFile->QueryInfo(HTTP_QUERY_CONTENT_LENGTH | 
							 HTTP_QUERY_FLAG_NUMBER, len1);
			len2 = (int)pFile->GetLength();
			// Take the max.
			len = __max(len1, len2);
			// Last resort.
			if(len < 1024)
			{
				TRACE(_T("Video File Length is %d - estimating new len\n"), len);
				len = _ttoi(video.duration);	// seconds
				len *= 85000;					// 85000 = mucho data per second.
			}
			TRACE(_T("Video File Length %d\n"), len);

			// Set Video Progress (progress2).
			stepSz = len / 100;					// 100 steps
			step = 0;
			vidProgStep = 0;
			SetProgress2Range(100);
			SetProgress2Pos(0);

			// Total read.
			nTotalRead = 0;
			nReadKBps = 0;

			// KB per sec.
			startTime = GetTickCount();

			// Get chunks.
			while((nRead = pFile->Read(buf, bufSz)) > 0)
			{
				destfile.Write(buf, nRead);

				// Display KBps - once a second.
				nReadKBps += nRead;
				endTime = GetTickCount();
				if(endTime > startTime + 1000)
				{
					KBps = endTime - startTime;				
					// 1 KBps (KB/s) = 1,000 bytes per second;
					KBps = (double)(nReadKBps / KBps);
					SetKBps(KBps);
					
					// New start time for next KBps.
					startTime = endTime;
					nReadKBps = 0;
				}

				// Step the Video Progress ctrl.
				step += nRead;
				if(step > stepSz)
				{				
					// Update the progress.
					vidProgStep ++;
					SetProgress2Pos(vidProgStep);
					step = 0;
				}

				// For total below.
				nTotalRead += nRead;				

				// User can abort during download.
				if(::WaitForSingleObject(m_params.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
				{
					// Close and delete dest file.
					destfile.Close();
					destfile.Remove(destPath);

					if(pFile)
					{
						pFile->Close();
						delete pFile;
						pFile = NULL;
					}
					Close();

					// Post Status msg.
					msg = _T("Canceling Video Download.");
					ShowStatus(msg);

					// Info user if partial download.
					if(i > 0)
					{
						msg.Format(_T("%d of %d videos downloaded."), i, sz);
						ShowStatus(msg);
					}

					TRACE(_T("CKnanInet GetVideoFiles user aborting\n"));
					// Gone.
					return FALSE;
				}
			}

			// Encountered one video with a total size of 290 bytes.
			// VLC rejets it if streaming, and it just gives a flash
			// if played from disk. So lets reject videos with less
			// than 500 bytes - see what happens. An mp4 must have a 
			// header of some size which could be used instead.
			if(nTotalRead < 500)
			{
				// Info user.
				msg.Format(_T("Video file corrupted. Discarding."));
				ShowStatus(msg);

				// Close and delete the bogus file.
				destfile.Close();
				destfile.Remove(destPath);

				// Clean up the stream / connection.
				if(pFile)
				{
					pFile->Close();
					delete pFile;
					pFile = NULL;
				}
				// Max the Video Progress.
				SetProgress2Pos(100);

				// Increment the Total Progress.
				SetProgressPos( (i + 1) * 10);

				// Increment for final message.
				badFiles ++;

				// Get next
				continue;
			}

			// File downloaded.
			destfile.Close();

			// Max the Video Progress.
			SetProgress2Pos(100);

			// Update the Total Progress.
			SetProgressPos( (i + 1) * 10);
		
			// Calc video sz in MB.
			double mbRead = (double)nTotalRead;
			mbRead /= (1024 * 1024);

			// Print num downloaded and sz.		 
			msg.Format(_T("  %d of %d Downloaded - %.1f MB"), i + 1, sz, mbRead);
			ShowStatus(msg);

			// Clean up the stream / connection.
			if(pFile)
			{
				pFile->Close();
				delete pFile;
				pFile = NULL;
			}
		}

		// Done downloading - close the connection.
		Close();
	}

     // Catch any exceptions from WinINet      
	catch(CInternetException * pEx) 
	{
		// 12/13/14
		// Note:
		// Error code 12002 (pEx->m_dwError) is a timeout controlled 
		// by the INTERNET_OPTION_RECEIVE_TIMEOUT option.
		// Currently we cancel the whole download operation.
		// How to do otherwise??

		TCHAR szErr[1024];
        if(!pEx->GetErrorMessage(szErr, 1024))
			//wcscpy(szErr,_T("Unknown error."));
			StringCbCopy(szErr, 15 * sizeof(TCHAR), _T("Unknown error."));

		TRACE(_T("CKnanInet GetWebFile File transfer failed!! - %s"),szErr);      
		pEx->Delete();

 		// Close and delete any opened dest file.
		CFileStatus status;
		if(CFile::GetStatus(destPath, status)) // static version for test.
		{
			TRACE(_T("CKnanInet GetVideoFiles exception - deleting video file\n"));
			destfile.Close();
			destfile.Remove(destPath);
		}

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}
		// Derived
		Close(); 

		// Info user with exception error.
		msg = szErr;
		msg.TrimRight();	// has a newline
		msg += _T(". ");	// beautify
		ShowStatus(msg);

		// Info user if not connected.
		if(msg == _T("The server name or address could not be resolved. "))
		{
			msg = _T("Please ensure you are connected to the internet.");
			ShowStatus(msg);
		}

		// Info user of failure.
		msg = _T("Video download failed.");
		ShowStatus(msg);

		// Info user if partial download.
		if(i > 0)
		{
			msg.Format(_T("%d of %d videos downloaded."), i, sz);
			ShowStatus(msg);
		}

		return FALSE;
	}

	// Finish Success.
	msg.Format(_T("%d of %d Videos downloaded successfully."), i - badFiles, sz);
	ShowStatus(msg);

	return TRUE;
}

// Updated 11/05/2017 - check if update needed logic'
//	todo - verify - set up folder - see note below
//
// Return 1 for update downloaded and ready to be installed.
// Return 0 for no update needed.
// Return -1 for error.
//
// 5/26/2017 SourceForge is now HTTPS.
// 03Serv reports "An error occurred in the secure channel support"
// and cannot be fixed. See HTTPS_SourceForge.txt.
// Does work on win7/10 with no changes however.
// Added error msg in catch().
//
BOOL CKnanInet::GetAppUpdate()
{
	// If hosted at TUX (defined in StdAfx.h).
	if(TUX_HOST)
		return GetAppUpdateTux();

	// 11/05/2017
	//
	// IMPORTANT:
	// If the Version.dat major update flag is set true, all versions 
	// which check that patch folder need to download the major update,
	// and that patch folder must be retired (not modified).
	// Therefore, the major update app must also have a new patch 
	// folder on the server for future updates, and be hardcoaded to 
	// it in the paths below.  
	//
	// If the major update flag is not set true, the current 
	// patch folder path below may be reused.
	//
	// Major update 0.9.6.7. New patch folder path: patch_0.9.5.7_ds
	CString pathVersion = _T("http://sourceforge.net/projects/khandesktop/files/")
						  _T("SourceCode/patch_0.9.5.7/Version.dat/download");
	CString pathKhan = _T("http://sourceforge.net/projects/khandesktop/files")
					   _T("/SourceCode/patch_0.9.5.7/Khan.dat/download");
	
	CString headers	= _T("Accept: */*\r\n");

	DWORD flags =	INTERNET_FLAG_TRANSFER_BINARY  |
					INTERNET_FLAG_EXISTING_CONNECT |					  
			//		INTERNET_FLAG_NO_AUTO_REDIRECT |	// allow SF.net redirect to SF.net mirror
					INTERNET_FLAG_DONT_CACHE;

	// Set derived CInternetSession session options.
	SetOption(INTERNET_OPTION_CONNECT_TIMEOUT,	30000);
	SetOption(INTERNET_OPTION_RECEIVE_TIMEOUT,	60000);	
	SetOption(INTERNET_OPTION_SEND_TIMEOUT,		60000);
	SetOption(INTERNET_OPTION_CONNECT_RETRIES,	1);

	CHttpFile * pFile = NULL;
	DWORD dwRet = 0;
	CString msg, s; //, sfPath;
	const int bufSz = 4096;
	char buf[bufSz];
	int nRead, nTotalRead;

	// Init string member to current text set in AppUpdateDialog.
	((CApp*)AfxGetApp())->GetThreadMessage(m_strStatus);

	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		msg = _T("Internal error during application update.");
		ShowStatus(msg);
		TRACE(_T("ERROR CKnanInet GetAppUpdateNeeded no pDoc\n"));
		Close();
		return -1;		
	}

	try 
	{
		// Update status msg.
		msg.Format(_T("Checking for updates: Please wait.")); 
		ShowStatus(msg);

		// Open URL.
		pFile = (CHttpFile*)OpenURL(pathVersion,
									1, 
									flags,
									headers, 
									-1);
		// Error.
		if(!pFile)
		{
			msg = _T("Error connecting to SourceForge.");
			ShowStatus(msg);
			TRACE(_T("ERROR CKnanInet GetAppUpdateNeeded pFile null\n"));
			Close();
			return -1;
		}

		// Get chunks.
		nTotalRead = 0;
		TCHAR ch;
		CString str;
		CStringArray strAr;
		while((nRead = pFile->Read(buf, bufSz)) > 0)
		{
			nTotalRead += nRead;

			// User can abort during download.
			if(::WaitForSingleObject(m_params.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
			{
				// Close connection.
				if(pFile)
				{
					pFile->Close();
					delete pFile;
					pFile = NULL;
				}
				// Close session.
				Close();

				// Post Status msg.
				msg = _T("Canceling Application Update.");
				ShowStatus(msg);

				TRACE(_T("CKnanInet GetAppUpdate user aborting\n"));

				// Bail.
				return -1;
			}
			
			// Parse the chunk into strings.
			for(int i = 0; i < nRead; i++)
			{
				ch = buf[i];

				// If a carrige return.
				if(ch == 13)
					continue;

				// If a line feed - \n'
				if(ch == 10)
				{
					if(!str.IsEmpty())
					{
						// If the requested file is not avialable, sf.net may
						// return a web page. The first string will confirm.
						if(strAr.GetSize() == 0 &&
						   str != UPDATE_START_STR) // "khandesktopupdate"
						{
							// Clean up the stream / connection.
							if(pFile)
							{
								pFile->Close();
								delete pFile;
								pFile = NULL;
							}
							// Done downloading - close the connection.
							Close();
							// Message user.
							msg = _T("An update cannot be found on the server.");
							ShowStatus(msg);
							TRACE(_T("ERROR No source forge patch file exists.\n"));
							Close();
							return -1;		
						}
						strAr.Add(str);
						str.Empty();
					}
					continue;
				}
				str += ch;
			}
		}
		// Get last string.
		if(!str.IsEmpty())
			strAr.Add(str);

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		// Ensure strings gotten.
		int sz = strAr.GetSize();
		if(sz < 3)
		{
			msg = _T("Update failed - data incomlete.");
			ShowStatus(msg);
			TRACE(_T("ERROR CKnanInet GetAppUpdateNeeded not 2 strings\n"));
			Close();
			return -1;		
		}
		// Check the version.
		CString newVersion = strAr[1];
		CString oldVersion = *m_params.version;
		//
		// Returns  1 if v1 > v2.	(new Version >  current Version)
		// Returns  0 if v1 <= v2.  (new Version <= current Version)
		// Returns -1 an error.
		int ret = pDoc->CompareVersions(newVersion, oldVersion);

		// rework 11/05/2017
		//
		BOOL majorNeeded = FALSE;
		if( sz > 3 && strAr[3] == _T("true") ) 
			majorNeeded = TRUE;
		BOOL downloadNeeded = TRUE;

		// Error
		if(ret == -1)
		{
			msg = _T("Update failed - version info error.");
			downloadNeeded = FALSE;
		}
		// Major update required. See patch folder notes above for new release!
		else if(majorNeeded)
		{
			msg = _T("\r\nA Major Update is available at\r\n")
				  _T("www.sourceforge.net/projects/khandesktop/\r\n");
			downloadNeeded = FALSE;
		}
		// Update not needed.
		else if(ret == 0)
		{
			msg = _T("You have the latest version!");
			downloadNeeded = FALSE;
		}
		// No update needed.
		if(!downloadNeeded)
		{
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdateTux %s\n"), msg);
			Close();
			return 0; // no update needed
		}

		// Else update available - download.
		//
		// Get the exeSz.
		str = strAr[2];
		str.TrimLeft();
		str.TrimRight();
		int exeSz = _wtoi(str); 

		// Download the exe to application data folder.

		// Open URL.
		pFile = (CHttpFile*)OpenURL(pathKhan,
									1, 
									flags,
									headers, 
									-1);
		// Error.
		if(!pFile)
		{
			msg = _T("Update failed - error obtaining update file.");
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdate pFile null\n"));
			Close();
			return -1;
		}

		// File for writing - temp file created in AppDataFolder - m_params.path.
		//   VC6    - D:\Programming(D)\Workspaces\MFC Workspaces\Khan\Khan_new.exe
		//	 VS2005 - C:\Documents and Settings\All Users\Application Data\KhanDesktop
		//	 See: CDoc::CleanCanceledAppUpdate() and CApp::GetAppDataFolder()
		CFile destfile;
		CString destPath = *m_params.path;
		CFileException ex;
		if(!destfile.Open(destPath, CFile::modeCreate |
									CFile::modeWrite  |
									CFile::typeBinary,
									&ex				  ))
		{
			TCHAR szError[1024];
			ex.GetErrorMessage(szError, 1024);
			msg.Format(_T("Update failed - error creating update file\n%s."), szError);
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdate can't open destFile\n"));
			if(pFile)
			{
				pFile->Close();
				delete pFile;
				pFile = NULL;
			}
			Close();
			return -1;
		}

		// Update status msg.
		msg.Format(_T("Downloading Update data: Please wait.")); 
		ShowStatus(msg);

		// Get chunks.
		nTotalRead = 0;
		while((nRead = pFile->Read(buf, bufSz)) > 0)
		{
			destfile.Write(buf, nRead);
			nTotalRead += nRead;

			// User can abort during download.
			if(::WaitForSingleObject(m_params.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
			{
				// Close connection.
				if(pFile)
				{
					pFile->Close();
					delete pFile;
					pFile = NULL;
				}
				// Close session.
				Close();

				// Close and delete file.
				destfile.Close();
				CFile::Remove(destPath);

				// Post Status msg.
				msg = _T("Canceling Application Update.");
				ShowStatus(msg);

				TRACE(_T("CKnanInet GetAppUpdate user aborting file download\n"));

				// Bail.
				return -1;
			}
		}

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		// File downloaded 0k.
		destfile.Close();

		// Done downloading - close the connection.
		Close();

		// Verify download size - see GetAppUpdateNeeded().
		if(nTotalRead != exeSz)
		{
			// Delete downloaded file.
			CFile::Remove(destPath);

			msg = _T("Update failed - update file is corrupt.");
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdate file is wrong size\n"));
			return -1;
		}

		// Update needed.
		msg = _T("\r\nAn update is available.");
		ShowStatus(msg);
		msg = _T("\r\nPress the Install Update button to install.");
		ShowStatus(msg);
	//	msg = _T("Note: An administrator will be needed to install the update.");
	//	ShowStatus(msg);
		TRACE(_T("CKnanInet GetAppUpdate ok to update\n"));
		return 1;
	}

     // Catch any exceptions from WinINet. Note this will 
	 // have occured befor any temp file is written.
	catch(CInternetException * pEx) 
	{
		// 5/26/2017
		DWORD dwRet = pEx->m_dwError;	// 12157 ERROR_WINHTTP_SECURE_CHANNEL_ERROR

		TCHAR szErr[1024];
        if(!pEx->GetErrorMessage(szErr, 1024))
			//wcscpy(szErr,_T("Unknown error."));
			StringCbCopy(szErr, 15 * sizeof(TCHAR), _T("Unknown error."));

		TRACE(_T("CKnanInet GetAppUpdate exception - %s"),szErr);      
		pEx->Delete(); 		

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}
		// Derived
		Close(); 

		// Info user with exception error.
		msg = szErr;
		msg.TrimRight();	// has a newline
		msg += _T(". ");	// beautify
		ShowStatus(msg);

		// 5/26/2017
		//  "An error occurred in the secure channel support"
		if(dwRet == 12157)	// ERROR_WINHTTP_SECURE_CHANNEL_ERROR
		{					
			msg = _T("Please check for a newer version at\r\n")
				  _T("www.sourceforge.net/projects/khandesktop/.");
			ShowStatus(msg);
		}

		// Info user if not connected.
		if(msg == _T("The server name or address could not be resolved. "))
		{
			msg = _T("Please ensure you are connected to the internet.");
			ShowStatus(msg);
		}

		// Info user of failure.
		msg = _T("Check for Updates failed.\r\n");
		ShowStatus(msg);

		return -1;
	}

	// Should not get here.
	TRACE(_T("ERROR CKnanInet GetAppUpdate - end of function\n")); 
	return -1;	
}

// Updated 11/05/2017 - check if update needed logic
// Get an update - for TuxFamily.org hosting - 10/05/14
//
// Return 1 for update downloaded and ready to be installed.
// Return 0 for no update needed.
// Return -1 for error.
//
BOOL CKnanInet::GetAppUpdateTux()
{
	// 11/05/2017
	//
	// IMPORTANT:
	// If the Version.dat major update flag is set true, all versions 
	// which check that patch folder need to download the major update,
	// and that patch folder must be retired (not modified).
	// Therefore, the major update app must also have a new patch 
	// folder on the server for future updates, and be hardcoaded to 
	// it in the paths below.  
	//
	// If the major update flag is not set true, the current 
	// patch folder path below may be reused.
	//
	// Major update 0.9.6.7. New patch folder path: patch_0.9.6.7_ds
	CString pathVersion = 
		_T("http://download.tuxfamily.org/khandesktop/Patches/patch_0.9.6.7_ds/Version.dat");

	// Tux Path for Khan.dat.
	CString pathKhan = 
		_T("http://download.tuxfamily.org/khandesktop/Patches/patch_0.9.6.7_ds/Khan.dat");

	CString headers	= _T("Accept: */*\r\n");

	DWORD flags =	INTERNET_FLAG_TRANSFER_BINARY  |
					INTERNET_FLAG_EXISTING_CONNECT |					  
			//		INTERNET_FLAG_NO_AUTO_REDIRECT |	// allow redirect to mirror?
					INTERNET_FLAG_DONT_CACHE;

	// Set derived CInternetSession session options.
	SetOption(INTERNET_OPTION_CONNECT_TIMEOUT,	30000);
	SetOption(INTERNET_OPTION_RECEIVE_TIMEOUT,	60000);	
	SetOption(INTERNET_OPTION_SEND_TIMEOUT,		60000);
	SetOption(INTERNET_OPTION_CONNECT_RETRIES,	1);

	CHttpFile * pFile = NULL;
	DWORD dwRet = 0;
	CString msg, s;
	const int bufSz = 4096;
	char buf[bufSz];
	int nRead, nTotalRead;

	// Init string member to current text set in AppUpdateDialog.
	((CApp*)AfxGetApp())->GetThreadMessage(m_strStatus);

	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		msg = _T("Internal error during application update.");
		ShowStatus(msg);
		TRACE(_T("ERROR CKnanInet GetAppUpdateNeeded no pDoc\n"));
		Close();
		return -1;		
	}

	try 
	{
		// Update status msg.
		msg.Format(_T("Checking for updates: Please wait.")); 
		ShowStatus(msg);

		// Open URL to version file.
		pFile = (CHttpFile*)OpenURL(pathVersion,
									1, 
									flags,
									headers, 
									-1);
		// Error.
		if(!pFile)
		{
			msg = _T("Error connecting to TuxFamily.org.");
			ShowStatus(msg);
			TRACE(_T("ERROR CKnanInet GetAppUpdateNeeded pFile null\n"));
			Close();
			return -1;
		}

		// Get chunks.
		nTotalRead = 0;
		TCHAR ch;
		CString str;
		CStringArray strAr;
		while((nRead = pFile->Read(buf, bufSz)) > 0)
		{
			nTotalRead += nRead;

			// User can abort during download.
			if(::WaitForSingleObject(m_params.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
			{
				// Close connection.
				if(pFile)
				{
					pFile->Close();
					delete pFile;
					pFile = NULL;
				}
				// Close session.
				Close();

				// Post Status msg.
				msg = _T("Canceling Application Update.");
				ShowStatus(msg);

				TRACE(_T("CKnanInet GetAppUpdate user aborting\n"));

				// Bail.
				return -1;
			}
			
			// Parse the chunk into strings.
			for(int i = 0; i < nRead; i++)
			{
				ch = buf[i];

				// If a carrige return.
				if(ch == 13)
					continue;

				// If a line feed - \n'
				if(ch == 10)
				{
					if(!str.IsEmpty())
					{
						// If the requested file is not avialable, tux may
						// return a web page. The first string will confirm.
						if(strAr.GetSize() == 0 &&
						   str != UPDATE_START_STR) // "khandesktopupdate"
						{
							// Clean up the stream / connection.
							if(pFile)
							{
								pFile->Close();
								delete pFile;
								pFile = NULL;
							}
							// Done downloading - close the connection.
							Close();
							// Message user.
							msg = _T("An update cannot be found on the server.");
							ShowStatus(msg);
							TRACE(_T("ERROR No TuxFamily.org patch file exists.\n"));
							Close();
							return -1;		
						}
						strAr.Add(str);
						str.Empty();
					}
					continue;
				}
				str += ch;
			}
		}
		// Get last string.
		if(!str.IsEmpty())
			strAr.Add(str);

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		// Ensure strings gotten.
		int sz = strAr.GetSize();
		if(sz < 3)
		{
			msg = _T("Update failed - data incomplete.");
			ShowStatus(msg);
			TRACE(_T("ERROR CKnanInet GetAppUpdateNeeded not 2 strings\n"));
			Close();
			return -1;		
		}
		// Compare the versions.
		CString newVersion = strAr[1];
		CString oldVersion = *m_params.version;
		//
		// Returns  1 if v1 > v2.	(new Version >  current Version)
		// Returns  0 if v1 <= v2.  (new Version <= current Version)
		// Returns -1 an error.
		int ret = pDoc->CompareVersions(newVersion, oldVersion);

		// rework 11/05/2017
		//
		// Flags.
		BOOL majorNeeded = FALSE;
		if( sz > 3 && strAr[3] == _T("true") ) 
			majorNeeded = TRUE;
		BOOL downloadNeeded = TRUE;

		// Error
		if(ret == -1)
		{
			msg = _T("Update failed - version info error.");
			downloadNeeded = FALSE;
		}
		// Major update required. See patch folder notes above for new release!
		else if(majorNeeded)
		{
			msg = _T("\r\nA Major Update is available at\r\n")
				  _T("www.khandesktop.tuxfamily.org\r\n");
			downloadNeeded = FALSE;
		}
		// Update not needed.
		else if(ret == 0)					// current Version >= new Version
		{
			msg = _T("You have the latest version!");
			downloadNeeded = FALSE;
		}
		// If no update needed.
		if(!downloadNeeded)
		{
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdateTux %s\n"), msg);
			Close();
			return 0; // 0 for no update needed
		}

		// Else update available - download: Current Version < new Version & no major needed
		//
		// Get the exeSz.
		str = strAr[2];
		str.TrimLeft();
		str.TrimRight();
		int exeSz = _wtoi(str); 

		// Download the exe to application data folder.
		//
		// Open URL.
		pFile = (CHttpFile*)OpenURL(pathKhan,
									1, 
									flags,
									headers, 
									-1);
		// Error.
		if(!pFile)
		{
			msg = _T("Update failed - error obtaining update file.");
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdate pFile null\n"));
			Close();
			return -1;
		}

		// File for writing - temp file created in AppDataFolder - m_params.path.
		//   VC6    - D:\Programming(D)\Workspaces\MFC Workspaces\Khan\Khan_new.exe
		//	 VS2005 - C:\Documents and Settings\All Users\Application Data\KhanDesktop
		//	 See: CDoc::CleanCanceledAppUpdate() and CApp::GetAppDataFolder()
		CFile destfile;
		CString destPath = *m_params.path; 
		CFileException ex;
		if(!destfile.Open(destPath, CFile::modeCreate |
									CFile::modeWrite  |
									CFile::typeBinary,
									&ex				  ))
		{
			TCHAR szError[1024];
			ex.GetErrorMessage(szError, 1024);
			msg.Format(_T("Update failed - error creating update file\n%s."), szError);
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdate can't open destFile\n"));
			if(pFile)
			{
				pFile->Close();
				delete pFile;
				pFile = NULL;
			}
			Close();
			return -1;
		}

		// Update status msg.
		msg.Format(_T("Downloading Update data: Please wait.")); 
		ShowStatus(msg);

		// Get chunks.
		nTotalRead = 0;
		while((nRead = pFile->Read(buf, bufSz)) > 0)
		{
			destfile.Write(buf, nRead);
			nTotalRead += nRead;

			// User can abort during download.
			if(::WaitForSingleObject(m_params.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
			{
				// Close connection.
				if(pFile)
				{
					pFile->Close();
					delete pFile;
					pFile = NULL;
				}
				// Close session.
				Close();

				// Close and delete file.
				destfile.Close();
				CFile::Remove(destPath);

				// Post Status msg.
				msg = _T("Canceling Application Update.");
				ShowStatus(msg);

				TRACE(_T("CKnanInet GetAppUpdate user aborting file download\n"));

				// Bail.
				return -1;
			}
		}

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}

		// File downloaded 0k.
		destfile.Close();

		// Done downloading - close the connection.
		Close();

		// Verify download size - see GetAppUpdateNeeded().
		if(nTotalRead != exeSz)
		{
			// Delete downloaded file.
			CFile::Remove(destPath);

			msg = _T("Update failed - update file is corrupt.");
			ShowStatus(msg);
			TRACE(_T("CKnanInet GetAppUpdate file is wrong size\n"));
			return -1;
		}

		// Update needed.
		msg = _T("\r\nAn update is available.");
		ShowStatus(msg);
		msg = _T("\r\nPress the Install Update button to install.");
		ShowStatus(msg);
		TRACE(_T("CKnanInet GetAppUpdate ok to update\n"));
		return 1;
	}

     // Catch any exceptions from WinINet. Note this will 
	 // have occured befor any temp file is written.
	catch(CInternetException * pEx) 
	{
		TCHAR szErr[1024];
        if(!pEx->GetErrorMessage(szErr, 1024))
			//wcscpy(szErr,_T("Unknown error."));
			StringCbCopy(szErr, 15 * sizeof(TCHAR), _T("Unknown error."));

		TRACE(_T("CKnanInet GetAppUpdate exception - %s"),szErr);      
		pEx->Delete(); 		

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}
		// Derived
		Close(); 

		// Info user with exception error.
		msg = szErr;
		msg.TrimRight();	// has a newline
		msg += _T(". ");	// beautify
		ShowStatus(msg);

		// 10/04/2017 - copied fron sf version above in case
		// a future https errpr.
		//  "An error occurred in the secure channel support"
		if(dwRet == 12157)	// ERROR_WINHTTP_SECURE_CHANNEL_ERROR
		{					
			msg = _T("Please check for a newer version at\r\n")
				  _T("www.khandesktop.tuxfamily.org.");
			ShowStatus(msg);
		}


		// Info user if not connected.
		if(msg == _T("The server name or address could not be resolved. "))
		{
			msg = _T("Please ensure you are connected to the internet.");
			ShowStatus(msg);
		}

		// Info user of failure.
		msg = _T("Check for Updates failed.\r\n");
		ShowStatus(msg);

		return -1;
	}

	// Should not get here.
	TRACE(_T("ERROR CKnanInet GetAppUpdate - end of function\n")); 
	return -1;	
}

////////////////////////////////////////////
// Constructor for internet connection test.

// 12/13/14 not used - param version for threads used instead
CKnanInet::CKnanInet()

			 : CInternetSession(_T("Khan Desktop Application"), // App name
								1,								// dwContext: 1 = use callback
								PRE_CONFIG_INTERNET_ACCESS)
																// dwFlags INTERNET_FLAG_ASYNC			 
{
	// Enable status callbacks
	EnableStatusCallback(TRUE);
}

/////////////////////////////////////////////////////////////
// Make an internet connection test for video file download.
// Example url: 
//	"http://s3.amazonaws.com/KA-youtube-converted/frBJEYvyd-8.mp4/frBJEYvyd-8.mp4"
//
// 05/15 - Fastly now fixed. Change timeout to 5 sec.
// 
BOOL CKnanInet::TestInetStream(CString url, CString & errMsg)
{
	// Prepare.
	CString headers	= _T("Accept: */*\r\n");
	DWORD flags =	INTERNET_FLAG_TRANSFER_BINARY  |
					INTERNET_FLAG_EXISTING_CONNECT |					  
					INTERNET_FLAG_NO_AUTO_REDIRECT | 
					INTERNET_FLAG_DONT_CACHE;
	SetOption(INTERNET_OPTION_CONNECT_TIMEOUT, 5000); // 05/15

	CHttpFile * pFile = NULL;
	const int bufSz = 1024;
	char buf[bufSz];
	try 
	{
		BOOL ret = FALSE;
		// Open URL.
		pFile = (CHttpFile*)OpenURL(url,
									1, 
									flags,
									headers, 
									-1);
		// Error.
		if(!pFile)
		{
			errMsg = _T("Error connecting to internet.");
			return FALSE;
		}

		// Test read.
		int nRead = pFile->Read(buf, bufSz);

		// Ok.
		if(nRead)
			ret = TRUE;
		// Error.
		else
		{
			ret = FALSE;
			errMsg = _T("Could not get data from the internet.");

		}
		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}
		Close();
		return ret;
	}

     // Catch any exceptions from WinINet      
	catch(CInternetException * pEx) 
	{
		TCHAR szErr[1024];
        if(!pEx->GetErrorMessage(szErr, 1024))
			//wcscpy(szErr,_T("Unknown error."));
			StringCbCopy(szErr, 15 * sizeof(TCHAR), _T("Unknown error."));

		TRACE(_T("CKnanInet TestInetStream failed! - %s"),szErr);      
		pEx->Delete();

		// Clean up the stream / connection.
		if(pFile)
		{
			pFile->Close();
			delete pFile;
			pFile = NULL;
		}
		// Derived
		Close(); 

		// Set err message.
		errMsg = szErr;
		errMsg.TrimRight();	// has a newline
		errMsg += _T(". ");	// beautify
		// If no connection error.
		if(errMsg == _T("The server name or address could not be resolved. "))
		{
			errMsg = _T("The server name or address could not be resolved.\n")
					 _T("Please ensure you are connected to the internet.");
		}
	}

	return FALSE;
}

// Callback
// See list at: http://support.microsoft.com/kb/943891
void CKnanInet::OnStatusCallback(DWORD dwContext, DWORD dwInternetStatus,
								 LPVOID lpvStatusInformation, 
								 DWORD dwStatusInformationLength)
{
	// Note: If Mfc TRACE Internet Client Tracing is checked
	// these will be traced.

    // Status callbacks need thread-state protection. 
    AFX_MANAGE_STATE(AfxGetAppModuleState( ));

    CString strStatus;

	switch (dwInternetStatus)
	{
	case INTERNET_STATUS_HANDLE_CREATED:
	case INTERNET_STATUS_RECEIVING_RESPONSE:
	case INTERNET_STATUS_RESPONSE_RECEIVED:
	case INTERNET_STATUS_HANDLE_CLOSING:
 	case INTERNET_STATUS_NAME_RESOLVED:
	case INTERNET_STATUS_CONNECTED_TO_SERVER:
	case INTERNET_STATUS_CLOSING_CONNECTION:
	case INTERNET_STATUS_CONNECTION_CLOSED:
       return;

	case INTERNET_STATUS_RESOLVING_NAME:
		strStatus = _T("Finding Server");
		break;

	case INTERNET_STATUS_CONNECTING_TO_SERVER:
		strStatus = _T("Connecting to Server");
		break;

	case INTERNET_STATUS_SENDING_REQUEST:
		strStatus = _T("Sending request");
		break;

	case INTERNET_STATUS_REQUEST_SENT:
		strStatus = _T("Request sent. Please wait...");
		break;

	case INTERNET_STATUS_REQUEST_COMPLETE:
  		strStatus = _T("Request complete");
		break;

	case INTERNET_STATUS_REDIRECT:
		strStatus = _T("Server redirecting");
		break;

	default:
		{
			if(dwInternetStatus != 320)
				strStatus.Format(_T("Status: %d"), dwInternetStatus);
			break;
		}
	}
	// Note: no callbacks displayed during GetVideoFiles().
	TRACE(_T("In CKnanInet OnStatusCallback calling ShowStatus\n"));

    ShowStatus(strStatus);
}

// Progress control range message to dialog.
void CKnanInet::SetProgressRange(int upper)
{
	// Post to dialog.
	if(m_params.hwndDlg)
		PostMessage(m_params.hwndDlg, WM_USER_PROGRESS_MAX, 0, (LPARAM)upper);
}

// Progress control position message to dialog.
void CKnanInet::SetProgressPos(int pos)
{
	// Post to dialog.
	if(m_params.hwndDlg)
		PostMessage(m_params.hwndDlg, WM_USER_PROGRESS_POS, 0, (LPARAM)pos);
}

// Send KBps values in message to dialog.
void CKnanInet::SetKBps(double KBps)
{
	// Post to dialog.
	if(m_params.hwndDlg)
	{
		double whole, fract;
		fract = modf(KBps, &whole);
		fract *= 10;
	//	TRACE(_T("whole = %.4f fract = %.4f\n"), whole, fract);
		PostMessage(m_params.hwndDlg, WM_USER_KBPS, (WPARAM)(int)whole, (LPARAM)(int)fract);
	}
}

// 2nd Progress control range message to dialog.
void CKnanInet::SetProgress2Range(int upper)
{
	// Post to dialog.
	if(m_params.hwndDlg)
		PostMessage(m_params.hwndDlg, WM_USER_PROGRESS2_MAX, 0, (LPARAM)upper);
}

// 2nd Progress control position message to dialog.
void CKnanInet::SetProgress2Pos(int pos)
{
	// Post to dialog.
	if(m_params.hwndDlg)
		PostMessage(m_params.hwndDlg, WM_USER_PROGRESS2_POS, 0, (LPARAM)pos);
}

// Status String message to dialog.
void CKnanInet::ShowStatus(CString strStatus)
{
	// Concat.
	if(!m_strStatus.IsEmpty())
		m_strStatus += _T("\r\n");
	m_strStatus += strStatus;

	// Set CApp string buff and Post message to dialog.
	if(m_params.hwndDlg)
	{
		((CApp*)AfxGetApp())->SetThreadMessage(m_strStatus);	// move to CMainFrame?
		PostMessage(m_params.hwndDlg, WM_USER_STATUS_STRING, 0, 0);
	}
}

/////////////////////////////////////////////////////////////////////////
// old - not used

// Covert UNIX text to DOS txt.
BOOL CKnanInet::Convert(CString unixPath, CString dosPath)
{
	if(unixPath == dosPath)
		return FALSE;

	FILE *fp1, *fp2;

	if((fp1 = _wfopen(unixPath, _T("rb"))) == NULL)
	{
		return FALSE;
	}
	if((fp2 = _wfopen(dosPath, _T("wt"))) == NULL)
	{
		fclose(fp1);
		return FALSE;
	}

	char buffer[4096];
	int charSz = sizeof(char);
	DWORD dwRead;

	// Read in 4096-byte blocks,
	do
	{
		// Read binary data to buff.
		dwRead = fread(buffer, charSz, 4096, fp1);

		// Write chunks - note fwrite will replace each carriage return 
		// with a carriage return / linefeed pair.
		fwrite(&buffer, charSz, dwRead, fp2);
	}
	while (dwRead > 0);

	// Clean up.
	fclose(fp1);
	fclose(fp2);

	return TRUE;
}

// Covert UNIX text to DOS txt.
BOOL CKnanInet::Convert2(CString unixPath, CString dosPath)
{
	FILE *in, *out;

	if((in = _wfopen(unixPath, _T("rb"))) == NULL)
	{
		return FALSE;
	}
	if((out = _wfopen(dosPath, _T("wt"))) == NULL)
	{
		fclose(in);
		return FALSE;
	}

	int ch, prev_ch= 0, rval = FALSE;

	const int LF		= 0x0A;	// line feed		10
	const int CR		= 0x0D;	// cariage return	13
	const int COMMA		= 0x2C;	// comma			44
	const int SPACE		= 0x20;	// space			32
	const int COLON		= 0x3A;	// colon			58
	const int SBRACE	= 0x7B;	// start brace		123
	const int EBRACE	= 0x7D;	// end brace		125
	const int SBRACKET	= 0x5B;	// start bracket	91
	const int EBRACKET	= 0x5D;	// end bracket		93
	
	
	CString str, prevStr;//dbg

	while( (ch = getc(in)) != EOF)
	{
	//	str = ch;//dbg
		
		switch(ch)
		{
		// Add CR/LF after commas. Note fopen "wt" translates LF to CR/LF.
		case COMMA:
			putc(ch, out);
			putc(LF, out);
			break;
		// Add CR/LF after a starting brace.
		case SBRACE:
			putc(ch, out);
			putc(LF, out);
			break;
		// Add CR/LF before an ending brace.
		case EBRACE:
			putc(LF, out);
			putc(ch, out);
			break;
		// Add CR/LF before and after a starting bracket.
		case SBRACKET:
			putc(LF, out);
			putc(ch, out);
			putc(LF, out);
			break;
		// Add CR/LF before and after an ending bracket.
		case EBRACKET:
			putc(LF, out);
			putc(ch, out);
			putc(LF, out);
			break;
		// Skip any existing CR or LF.
		case LF:
		case CR:
			continue;
		default:
			putc(ch, out);
			break;
		}

	/*	if(ch == LF)
		{
			if(prev_ch != CR)
			{
				putc(CR, out);
			}
		}
		else
			putc(ch, out);*/

		prev_ch = ch;
		prevStr = str;//dbg
	}
	
	if (fclose (in) == EOF)
	{
		rval = TRUE;
	}
	if (fclose (out) == EOF)
	{
		rval = TRUE;
	}
	// Done
	return rval;
}




